this.name           = "EquipmentSpace";
this.author         = "Ngalo";
this.copyright      = "(C) 2015 Me.";
this.licence        = "CC-NC-by-SA 2.0";
this.description    = "Script to manage the player ship (and potentially NPCs') equipment bays";
this.version        = "0.2";

"use strict";


this.startUp = this.playerBoughtNewShip = function()
{
	log(this.name, "(Re-)Initialising OXP " + this.name);

	this.$shipSetup(player.ship);
	this.$setPlayerEqIndicators();
	log(this.name + ": player ship configured")
}

//TODO for 1.81/1.82: change this handler to "equipmentAdded"
this.playerBoughtEquipment = function (equipment)
{
	if (equipment == "EQ_EQUIPMENTSPACE_EXTRA_REMOVAL")
	{
		for (var i = 0; i < player.ship.equipment.length; i++)
		{
			if (player.ship.equipment[i].name == "Extra Equipment Bay")
			{
				player.ship.removeEquipment(player.ship.equipment[i]);
				player.ship.removeEquipment("EQ_EQUIPMENTSPACE_EXTRA_REMOVAL");
			}
		}
	}

	var eqInfo = EquipmentInfo.infoForKey(equipment);
	var eqKeyWords = eqInfo.equipmentKey.split("_")

	if (eqKeyWords.pop() == "REMOVAL" && 0 < oolite.compareVersion("1.81"))
	{
		this.equipmentRemoved(eqKeyWords.join("_"))
	}
	else
	{
		this.$manageEquipmentAddition(eqInfo, player.ship);
	}
	this.$setPlayerEqIndicators();
}

this.equipmentRemoved = function (equipment)
{	//inefficient but works
	log("equipment removed: " + equipment)
	if (EquipmentInfo.infoForKey(equipment) && this.$getSpaceRequired(EquipmentInfo.infoForKey(equipment), player.ship))
	{
		this.$shipSetup(player.ship);
		this.$setPlayerEqIndicators();
		log(this.name + ": player ship reconfigured (equipment removal)")
	}
}

/*
Occasional problems with this handler. Commented out in favour of from-scratch reconfiguration.
this.equipmentRemoved = function (equipment)
{	
	if (equipment == "EQ_CARGO_BAY")
	{
		player.ship.EquipmentSpace_available += player.ship.EquipmentSpace_max;
		player.ship.EquipmentSpace_max += player.ship.EquipmentSpace_max;
	}
	else
	{	//NOT WORKING CORRECTLY?
		eqInfo = EquipmentInfo.infoForKey(equipment)
		player.ship.EquipmentSpace_available += this.$getSpaceRequired(equipment, player.ship);
		if (eqInfo.scriptInfo && eqInfo.scriptInfo.EquipmentSpace_is_conversion)
		{
			ship.EquipmentSpace_max += this.$getSpaceRequired(equipment, player.ship);
		}
	}
	this.$setPlayerEqIndicators();
}
*/

this.$getAllEquipment = function (ship)
{	// standard 'Ship.equipment' doesn't include weapons
	//TODO: include missiles (assume 0 space *unless specified*)?
	var eq = ship.equipment.concat([
					ship.forwardWeapon,
					ship.aftWeapon,
					ship.portWeapon,
					ship.starboardWeapon
				       ]);
	return eq;
}

this.$getSpaceRequired = function (equipment, ship)
{
	if (ship.scriptInfo && ship.scriptInfo["EquipmentSpace_for_"+equipment.equipmentKey])
	{
		return ship.scriptInfo["EquipmentSpace_for_"+equipment.equipmentKey];
	}
	if (equipment.scriptInfo && equipment.scriptInfo.EquipmentSpace_required)
	{
		return equipment.scriptInfo.EquipmentSpace_required;
	}
	if (equipment == EquipmentInfo.infoForKey("EQ_CARGO_BAY"))
	{	// special handling: 1/2 ship's max space, rounded up if necessary 
		return Math.ceil(ship.EquipmentSpace_max / 2);
	}
	/* this default is not currently used
	if (equipment.isVisible && equipment.damageProbability != 0)
	{	// default for all other visible, damageable equipment
		return 2;
	}
	*/
	else
	{	// default for anything else which doesn't define the relevant scriptInfo, or if a null object gets passed in (e.g. when removing "EQ_SOMETHING_DAMAGED" which has no actual EquipmentInfo)
		return 0;
	}
}

this.$manageEquipmentAddition = function (equipment, ship)
{	// change available or maximum space values when equipment is added
	if (equipment)
	{
		ship.EquipmentSpace_available -= this.$getSpaceRequired(equipment, ship);
		if (equipment.scriptInfo && equipment.scriptInfo.EquipmentSpace_is_conversion)
		{
			ship.EquipmentSpace_max -= this.$getSpaceRequired(equipment, ship);
		}
	}
}

//TODO: Manage equipment removal correctly- Done?

this.$setPlayerEqIndicators = function()
{	// set indicators in equipment list on F5 screen
	var eqUsed = player.ship.EquipmentSpace_max - player.ship.EquipmentSpace_available;
	for (var i=0; i<player.ship.equipment.length; i++)
	{
		var eqInfo = player.ship.equipment[i];
		var eqKeyWords = eqInfo.equipmentKey.split("_")
		if (eqKeyWords[1] == "EQUIPMENTSPACE")
		{
			if (eqKeyWords[2] == "USED" && eqKeyWords[3] != eqUsed)
			{
				player.ship.removeEquipment(eqInfo);
				i-- //ensure we don't accidentally skip an item, otherwise we sometimes get 2 different 'space used' indicators!
			}
			if (eqKeyWords[2] == "MAX" && eqKeyWords[3] != player.ship.EquipmentSpace_max)
			{
				player.ship.removeEquipment(eqInfo);
				i--
			}
		}
	}
	player.ship.awardEquipment("EQ_EQUIPMENTSPACE_USED_" + eqUsed);
	player.ship.awardEquipment("EQ_EQUIPMENTSPACE_MAX_" + player.ship.EquipmentSpace_max);
}

this.$shipSetup = function (ship)
{	//set up a ship's equipment bay correctly, accounting for what's already there

	if (ship.scriptInfo && ship.scriptInfo.EquipmentSpace_max)
	{	// if ship defines max space, set it up
		ship.EquipmentSpace_available = ship.EquipmentSpace_max = ship.scriptInfo.EquipmentSpace_max;
	}
	else
	{	// Default for OXP ships: slightly less than a Cobra 3
		ship.EquipmentSpace_available = ship.EquipmentSpace_max = 28;
	}

	var shipEq = this.$getAllEquipment(ship)
	for (var i=0; i < shipEq.length; i++)
	{
		var eqInfo = shipEq[i];
		//log("EquipmentSpace accounting for:" + eqInfo);
		//log("requires " + this.$getSpaceRequired(eqInfo, ship))

		this.$manageEquipmentAddition(eqInfo, ship);

		//log("space remaining" + ship.EquipmentSpace_available);
	}	
}	